@@ -8,7 +8,7 @@ import numpy
 import py
 import pytest
 
-import rtree
+from .common import sidx_version_string
 
 data_files = ["boxes_15x15.data"]
 
@@ -25,7 +25,7 @@ def temporary_working_directory(tmpdir: py.path.local) -> Iterator[None]:
 def pytest_report_header(config):
     """Header for pytest."""
     vers = [
-        f"SIDX version: {rtree.core.rt.SIDX_Version().decode()}",
+        f"SIDX version: {sidx_version_string}",
         f"NumPy version: {numpy.__version__}",
     ]
     return "\n".join(vers)
